#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import DatasetI


class Dataset201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Dataset'

    OMERO_CLASS = DatasetI

    def decode(self, data):
        v = super(Dataset201501Decoder, self).decode(data)
        self.set_property(v, 'name', data.get('Name'))
        self.set_property(v, 'description', data.get('Description'))
        for image in data.get('Images', list()):
            image_decoder = self.ctx.get_decoder(image['@type'])
            v.linkImage(image_decoder.decode(image))
        return v


class Dataset201606Decoder(Dataset201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Dataset'


if SCHEMA_VERSION == '2015-01':
    decoder = (Dataset201501Decoder.TYPE, Dataset201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Dataset201606Decoder.TYPE, Dataset201606Decoder)
DatasetDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import PlateAcquisitionI


class PlateAcquisition201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#PlateAcquisition'

    def encode(self, obj):
        v = super(PlateAcquisition201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'Name', obj.name)
        self.set_if_not_none(v, 'Description', obj.description)
        self.set_if_not_none(v, 'MaximumFieldCount', obj.maximumFieldCount)
        self.set_if_not_none(v, 'StartTime', obj.startTime)
        self.set_if_not_none(v, 'EndTime', obj.endTime)
        return v


class PlateAcquisition201606Encoder(PlateAcquisition201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#PlateAcquisition'


if SCHEMA_VERSION == '2015-01':
    encoder = (PlateAcquisitionI, PlateAcquisition201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (PlateAcquisitionI, PlateAcquisition201606Encoder)
PlateAcquisitionEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import ChannelI


class Channel201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Channel'

    def encode(self, obj):
        v = super(Channel201501Encoder, self).encode(obj)
        color = self.rgba_to_int(obj.red, obj.green, obj.blue, obj.alpha)
        self.set_if_not_none(v, 'Color', color)
        self.set_if_not_none(v, 'omero:lookupTable', obj.lookupTable)
        logical_channel = obj.logicalChannel
        if logical_channel is not None and logical_channel.isLoaded():
            self.set_if_not_none(
                v, 'omero:LogicalChannelId', logical_channel.id
            )
            self.set_if_not_none(
                v, 'EmissionWavelength', logical_channel.emissionWave
            )
            self.set_if_not_none(
                v, 'ExcitationWavelength', logical_channel.excitationWave
            )
            self.set_if_not_none(v, 'Fluor', logical_channel.fluor)
            self.set_if_not_none(v, 'Name', logical_channel.name)
            self.set_if_not_none(v, 'NDFilter', logical_channel.ndFilter)
            self.set_if_not_none(v, 'PinholeSize', logical_channel.pinHoleSize)
            self.set_if_not_none(
                v, 'PockelCellSetting', logical_channel.pockelCellSetting
            )
            self.set_if_not_none(
                v, 'SamplesPerPixel', logical_channel.samplesPerPixel
            )
            contrast_method = logical_channel.contrastMethod
            if contrast_method is not None and contrast_method.isLoaded():
                contrast_method_encoder = \
                    self.ctx.get_encoder(contrast_method.__class__)
                v['ContrastMethod'] = \
                    contrast_method_encoder.encode(contrast_method)
            illumination = logical_channel.illumination
            if illumination is not None and illumination.isLoaded():
                illumination_encoder = \
                    self.ctx.get_encoder(illumination.__class__)
                v['Illumination'] = \
                    illumination_encoder.encode(illumination)
            acquisition_mode = logical_channel.mode
            if acquisition_mode is not None and acquisition_mode.isLoaded():
                acquisition_mode_encoder = \
                    self.ctx.get_encoder(acquisition_mode.__class__)
                v['AcquisitionMode'] = \
                    acquisition_mode_encoder.encode(acquisition_mode)
            photometric_interpretation = \
                logical_channel.photometricInterpretation
            if photometric_interpretation is not None \
                    and photometric_interpretation.isLoaded():
                photometric_interpretation_encoder = \
                    self.ctx.get_encoder(photometric_interpretation.__class__)
                v['omero:photometricInterpretation'] = \
                    photometric_interpretation_encoder.encode(
                        photometric_interpretation
                    )
        return v


class Channel201606Encoder(Channel201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Channel'


if SCHEMA_VERSION == '2015-01':
    encoder = (ChannelI, Channel201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (ChannelI, Channel201606Encoder)
ChannelEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import ProjectI


class Project201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Project'

    def encode(self, obj):
        v = super(Project201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'Name', obj.name)
        self.set_if_not_none(v, 'Description', obj.description)
        if obj.isDatasetLinksLoaded() and obj.sizeOfDatasetLinks() > 0:
            datasets = list()
            for dataset_link in obj.copyDatasetLinks():
                dataset = dataset_link.child
                dataset_encoder = self.ctx.get_encoder(dataset.__class__)
                datasets.append(
                    dataset_encoder.encode(dataset)
                )
            v['Datasets'] = datasets
        return v


class Project201606Encoder(Project201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Project'


if SCHEMA_VERSION == '2015-01':
    encoder = (ProjectI, Project201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (ProjectI, Project201606Encoder)
ProjectEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import ChannelI, LogicalChannelI


class Channel201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Channel'

    OMERO_CLASS = ChannelI

    def decode(self, data):
        v = super(Channel201501Decoder, self).decode(data)
        color = data.get('Color')
        red, green, blue, alpha = self.int_to_rgba(color)
        self.set_property(v, 'red', red)
        self.set_property(v, 'green', green)
        self.set_property(v, 'blue', blue)
        self.set_property(v, 'alpha', alpha)
        self.set_property(v, 'lookupTable', data.get('omero:lookupTable'))

        logical_channel = LogicalChannelI(data.get('omero:LogicalChannelId'))
        logical_channel.emissionWave = \
            self.to_unit(data.get('EmissionWavelength'))
        logical_channel.excitationWave = \
            self.to_unit(data.get('ExcitationWavelength'))
        self.set_property(logical_channel, 'fluor', data.get('Fluor'))
        self.set_property(logical_channel, 'name', data.get('Name'))
        self.set_property(logical_channel, 'ndFilter', data.get('NDFilter'))
        logical_channel.pinHoleSize = self.to_unit(data.get('PinholeSize'))
        self.set_property(
            logical_channel, 'pockelCellSetting', data.get('PockelCellSetting')
        )
        self.set_property(
            logical_channel, 'samplesPerPixel', data.get('SamplesPerPixel')
        )
        contrast_method = data.get('ContrastMethod')
        if contrast_method is not None:
            contrast_method_decoder = self.ctx.get_decoder(
                contrast_method['@type']
            )
            logical_channel.contrastMethod = contrast_method_decoder.decode(
                contrast_method
            )
        illumination = data.get('Illumination')
        if illumination is not None:
            illumination_decoder = self.ctx.get_decoder(
                illumination['@type']
            )
            logical_channel.illumination = illumination_decoder.decode(
                illumination
            )
        acquisition_mode = data.get('AcquisitionMode')
        if acquisition_mode is not None:
            acquisition_mode_decoder = self.ctx.get_decoder(
                acquisition_mode['@type']
            )
            logical_channel.mode = acquisition_mode_decoder.decode(
                acquisition_mode
            )
        photometric_interpretation = \
            data.get('omero:photometricInterpretation')
        if photometric_interpretation is not None:
            photometric_interpretation_decoder = self.ctx.get_decoder(
                photometric_interpretation['@type']
            )
            logical_channel.photometricInterpretation = \
                photometric_interpretation_decoder.decode(
                    photometric_interpretation
                )
        v.logicalChannel = logical_channel

        return v


class Channel201606Decoder(Channel201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Channel'


if SCHEMA_VERSION == '2015-01':
    decoder = (Channel201501Decoder.TYPE, Channel201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Channel201606Decoder.TYPE, Channel201606Decoder)
PixelsDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from .. import Encoder
from omero.model import ExternalInfoI


class ExternalInfoEncoder(Encoder):

    TYPE = 'TBD#ExternalInfo'

    def encode(self, obj):
        v = super(ExternalInfoEncoder, self).encode(obj)
        if not obj.isLoaded():
            return v
        self.set_if_not_none(v, 'EntityId', obj.entityId)
        self.set_if_not_none(v, 'EntityType', obj.entityType)
        self.set_if_not_none(v, 'Lsid', obj.lsid)
        self.set_if_not_none(v, 'Uuid', obj.uuid)
        return v


encoder = (ExternalInfoI, ExternalInfoEncoder)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from .. import Decoder
from omero.model import ExternalInfoI


class ExternalInfoDecoder(Decoder):

    TYPE = 'TBD#ExternalInfo'

    OMERO_CLASS = ExternalInfoI

    def decode(self, data):
        v = super(ExternalInfoDecoder, self).decode(data)
        self.set_property(v, 'entityId', data.get('EntityId'))
        self.set_property(v, 'entityType', data.get('EntityType'))
        self.set_property(v, 'lsid', data.get('Lsid'))
        self.set_property(v, 'uuid', data.get('Uuid'))
        return v


decoder = (ExternalInfoDecoder.TYPE, ExternalInfoDecoder)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import Shape


class Shape201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#Shape'

    OMERO_CLASS = Shape

    def decode(self, data):
        v = super(Shape201501Decoder, self).decode(data)
        self.set_property(v, 'fillColor', data.get('FillColor'))
        self.set_property(v, 'fillRule', data.get('FillRule'))
        self.set_property(v, 'fontFamily', data.get('FontFamily'))
        v.fontSize = self.to_unit(data.get('FontSize'))
        self.set_property(v, 'fontStyle', data.get('FontStyle'))
        self.set_linecap(v, data)
        self.set_property(v, 'locked', data.get('Locked'))
        self.set_property(v, 'strokeColor', data.get('StrokeColor'))
        self.set_property(v, 'strokeDashArray', data.get('StrokeDashArray'))
        v.strokeWidth = self.to_unit(data.get('StrokeWidth'))
        self.set_property(v, 'textValue', data.get('Text'))
        self.set_property(v, 'theC', data.get('TheC'))
        self.set_property(v, 'theT', data.get('TheT'))
        self.set_property(v, 'theZ', data.get('TheZ'))
        self.set_visibility(v, data)
        transform = data.get('Transform')
        if transform:
            transform_decoder = self.ctx.get_decoder(transform['@type'])
            self.set_transform(v, transform_decoder.decode(transform))
        return v

    def set_visibility(self, v, data):
        self.set_property(v, 'visibility', data.get('Visible'))

    def set_linecap(self, v, data):
        self.set_property(v, 'strokeLineCap', data.get('LineCap'))

    def set_transform(self, v, transform):
        # For OMERO 5.1.x and OMERO 5.2.x, the transform is stored as a string
        # using the matrix(<a> <b> <c> <d> <e> <f>) style
        self.set_property(v, 'transform', str(transform))


class Shape201606Decoder(Shape201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Shape'

    def set_visibility(self, v, data):
        pass

    def set_linecap(self, v, data):
        pass

    def set_transform(self, v, transform):
        v.setTransform(transform)


if SCHEMA_VERSION == '2015-01':
    decoder = (Shape201501Decoder.TYPE, Shape201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Shape201606Decoder.TYPE, Shape201606Decoder)
ShapeDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .shape import ShapeDecoder
from omero.model import LineI


class Line201501Decoder(ShapeDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#Line'

    OMERO_CLASS = LineI

    def decode(self, data):
        v = super(Line201501Decoder, self).decode(data)
        self.set_property(v, 'x1', data.get('X1'))
        self.set_property(v, 'y1', data.get('Y1'))
        self.set_property(v, 'x2', data.get('X2'))
        self.set_property(v, 'y2', data.get('Y2'))
        return v


class Line201606Decoder(Line201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Line'

    def decode(self, data):
        v = super(Line201606Decoder, self).decode(data)
        self.set_property(v, 'markerStart', data.get('MarkerStart'))
        self.set_property(v, 'markerEnd', data.get('MarkerEnd'))
        return v


if SCHEMA_VERSION == '2015-01':
    decoder = (Line201501Decoder.TYPE, Line201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Line201606Decoder.TYPE, Line201606Decoder)
LineDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Decoder
from omero.model import Annotation


class Annotation201501Decoder(Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01#Annotation'

    OMERO_CLASS = Annotation

    def decode(self, data):
        v = super(Annotation201501Decoder, self).decode(data)
        self.set_property(v, 'description', data.get('Description'))
        self.set_property(v, 'ns', data.get('Namespace'))
        return v


class Annotation201606Decoder(Annotation201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Annotation'


class Annotatable201501Decoder(Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01#AnnotationRef'

    OMERO_CLASS = Annotation

    def decode(self, data):
        v = super(Annotatable201501Decoder, self).decode(data)
        if 'Annotations' in data:
            for annotation in data['Annotations']:
                annotation_decoder = self.ctx.get_decoder(annotation['@type'])
                v.linkAnnotation(annotation_decoder.decode(annotation))
        else:
            v.unloadAnnotationLinks()
        return v


class Annotatable201606Decoder(Annotatable201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#AnnotationRef'


if SCHEMA_VERSION == '2015-01':
    decoder = (Annotation201501Decoder.TYPE, Annotation201501Decoder)
    AnnotatableDecoder = Annotatable201501Decoder
elif SCHEMA_VERSION == '2016-06':
    decoder = (Annotation201606Decoder.TYPE, Annotation201606Decoder)
    AnnotatableDecoder = Annotatable201606Decoder
AnnotationDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Encoder

try:
    # Import transform classes introduced in OMERO 5.3.0
    from omero.model import AffineTransformI
except ImportError:
    # Use internal AffineTransformI classes for OMERO 5.1.x and OMERO 5.2.x
    from omero_marshal.legacy.affinetransform import AffineTransformI


class AffineTransform201501Encoder(Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#AffineTransform'

    def encode(self, obj):
        v = super(AffineTransform201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'A00', obj.getA00())
        self.set_if_not_none(v, 'A10', obj.getA10())
        self.set_if_not_none(v, 'A01', obj.getA01())
        self.set_if_not_none(v, 'A11', obj.getA11())
        self.set_if_not_none(v, 'A02', obj.getA02())
        self.set_if_not_none(v, 'A12', obj.getA12())
        return v


class AffineTransform201606Encoder(AffineTransform201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#AffineTransform'


if SCHEMA_VERSION == '2015-01':
    encoder = (AffineTransformI, AffineTransform201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (AffineTransformI, AffineTransform201606Encoder)
AffineTransformEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import ProjectI


class Project201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Project'

    OMERO_CLASS = ProjectI

    def decode(self, data):
        v = super(Project201501Decoder, self).decode(data)
        self.set_property(v, 'name', data.get('Name'))
        self.set_property(v, 'description', data.get('Description'))
        for dataset in data.get('Datasets', list()):
            dataset_decoder = self.ctx.get_decoder(dataset['@type'])
            v.linkDataset(dataset_decoder.decode(dataset))
        return v


class Project201606Decoder(Project201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Project'


if SCHEMA_VERSION == '2015-01':
    decoder = (Project201501Decoder.TYPE, Project201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Project201606Decoder.TYPE, Project201606Decoder)
ProjectDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Encoder
from omero.model import ExperimenterGroupI


class ExperimenterGroup201501Encoder(Encoder):

    TYPE = \
        'http://www.openmicroscopy.org/Schemas/OME/2015-01#ExperimenterGroup'

    def encode(self, obj):
        v = super(ExperimenterGroup201501Encoder, self).encode(obj)
        if not obj.isLoaded():
            return v
        self.set_if_not_none(v, 'Description', obj.description)
        self.set_if_not_none(v, 'Name', obj.name)
        return v


class ExperimenterGroup201606Encoder(ExperimenterGroup201501Encoder):

    TYPE = \
        'http://www.openmicroscopy.org/Schemas/OME/2016-06#ExperimenterGroup'


if SCHEMA_VERSION == '2015-01':
    encoder = (ExperimenterGroupI, ExperimenterGroup201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (ExperimenterGroupI, ExperimenterGroup201606Encoder)
ExperimenterGroupEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import ScreenI


class Screen201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#Screen'

    def encode(self, obj):
        v = super(Screen201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'Name', obj.name)
        self.set_if_not_none(v, 'Description', obj.description)
        self.set_if_not_none(v, 'ProtocolDescription', obj.protocolDescription)
        self.set_if_not_none(v, 'ProtocolIdentifier', obj.protocolIdentifier)
        self.set_if_not_none(v, 'ReagentSetDescription',
                             obj.reagentSetDescription)
        self.set_if_not_none(v, 'ReagentSetIdentifier',
                             obj.reagentSetIdentifier)
        self.set_if_not_none(v, 'Type', obj.type)
        if obj.isPlateLinksLoaded() and obj.sizeOfPlateLinks() > 0:
            plates = list()
            for plate_link in obj.copyPlateLinks():
                plate = plate_link.child
                plate_encoder = self.ctx.get_encoder(plate.__class__)
                plates.append(
                    plate_encoder.encode(plate)
                )
            v['Plates'] = plates
        return v


class Screen201606Encoder(Screen201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Screen'


if SCHEMA_VERSION == '2015-01':
    encoder = (ScreenI, Screen201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (ScreenI, Screen201606Encoder)
ScreenEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

# Needed to avoid import errors when this is the first import
import omero.all  # noqa
import omero.model
import omero.model.enums

import sys


class Decoder(object):

    TYPE = ''

    OMERO_CLASS = None

    def __init__(self, ctx):
        self.ctx = ctx

    def to_unit(self, v):
        if v is None:
            return None
        unit = v['@type'][v['@type'].rfind('#') + 1:]
        unit = getattr(omero.model, unit)
        return unit(
            float(v['Value']),
            v['Unit']
        )

    def set_property(self, target, prop, value):
        field_info = getattr(target._field_info, prop)
        if sys.version[0] == '2':
            try:
                if isinstance(value, unicode):
                    value = value.encode('utf-8')
            except NameError:
                pass
        setattr(
            target,
            prop,
            field_info.wrapper(
                value
            ) if value is not None else None
        )

    def decode(self, data):
        o = self.OMERO_CLASS(data.get('@id'))
        details = data.get('omero:details')
        if details is not None:
            decoder = self.ctx.get_decoder(details['@type'])
            o._details = decoder.decode(details)
        return o

    @classmethod
    def int_to_rgba(cls, rgba_int):
        """Converts a color Integer into r, g, b, a tuple."""
        if rgba_int is None:
            return None, None, None, None
        alpha = rgba_int % 256
        blue = rgba_int / 256 % 256
        green = rgba_int / 256 / 256 % 256
        red = rgba_int / 256 / 256 / 256 % 256
        return (red, green, blue, alpha)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Decoder
from omero.model import ExperimenterGroupI


class ExperimenterGroup201501Decoder(Decoder):

    TYPE = \
        'http://www.openmicroscopy.org/Schemas/OME/2015-01#ExperimenterGroup'

    OMERO_CLASS = ExperimenterGroupI

    def decode(self, data):
        v = super(ExperimenterGroup201501Decoder, self).decode(data)
        self.set_property(v, 'description', data.get('Description'))
        self.set_property(v, 'name', data.get('Name'))
        return v


class ExperimenterGroup201606Decoder(ExperimenterGroup201501Decoder):

    TYPE = \
        'http://www.openmicroscopy.org/Schemas/OME/2016-06#ExperimenterGroup'


if SCHEMA_VERSION == '2015-01':
    decoder = (ExperimenterGroup201501Decoder.TYPE,
               ExperimenterGroup201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (ExperimenterGroup201606Decoder.TYPE,
               ExperimenterGroup201606Decoder)
ExperimenterGroupDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .text_annotation import TextAnnotationDecoder
from omero.model import TagAnnotationI


class TagAnnotation201501Decoder(TextAnnotationDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01#TagAnnotation'

    OMERO_CLASS = TagAnnotationI

    def decode(self, data):
        v = super(TagAnnotation201501Decoder, self).decode(data)
        return v


class TagAnnotation201606Decoder(TagAnnotation201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#TagAnnotation'


if SCHEMA_VERSION == '2015-01':
    decoder = (TagAnnotation201501Decoder.TYPE,
               TagAnnotation201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (TagAnnotation201606Decoder.TYPE,
               TagAnnotation201606Decoder)
TagAnnotationDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotationDecoder
from omero.model import MapAnnotationI, NamedValue


class MapAnnotation201501Decoder(AnnotationDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01#MapAnnotation'

    OMERO_CLASS = MapAnnotationI

    def decode(self, data):
        v = super(MapAnnotation201501Decoder, self).decode(data)
        map_value = data.get('Value', list())
        map_value = [
            NamedValue(key, value) for key, value in map_value
        ]
        v.setMapValue(map_value)
        return v


class MapAnnotation201606Decoder(MapAnnotation201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#MapAnnotation'


if SCHEMA_VERSION == '2015-01':
    decoder = (MapAnnotation201501Decoder.TYPE,
               MapAnnotation201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (MapAnnotation201606Decoder.TYPE,
               MapAnnotation201606Decoder)
MapAnnotationDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Decoder
from omero.model import WellSampleI


class WellSample201501Decoder(Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#WellSample'

    OMERO_CLASS = WellSampleI

    def decode(self, data):
        v = super(WellSample201501Decoder, self).decode(data)
        v.posX = self.to_unit(data.get('PositionX'))
        v.posY = self.to_unit(data.get('PositionY'))
        self.set_property(v, 'timepoint', data.get('Timepoint'))
        image = data.get('Image')
        if image is not None:
            image_decoder = self.ctx.get_decoder(image['@type'])
            v.image = image_decoder.decode(image)
        plateacquisition = data.get('PlateAcquisition')
        if plateacquisition is not None:
            plateacq_decoder = self.ctx.get_decoder(plateacquisition['@type'])
            v.plateAcquisition = plateacq_decoder.decode(plateacquisition)
        return v


class WellSample201606Decoder(WellSample201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#WellSample'


if SCHEMA_VERSION == '2015-01':
    decoder = (WellSample201501Decoder.TYPE, WellSample201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (WellSample201606Decoder.TYPE, WellSample201606Decoder)
WellSampleDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import PlateI


class Plate201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#Plate'

    OMERO_CLASS = PlateI

    def decode(self, data):
        v = super(Plate201501Decoder, self).decode(data)
        self.set_property(v, 'name', data.get('Name'))
        self.set_property(v, 'description', data.get('Description'))
        self.set_property(v, 'columnNamingConvention',
                          data.get('ColumnNamingConvention'))
        self.set_property(v, 'rowNamingConvention',
                          data.get('RowNamingConvention'))
        self.set_property(v, 'columns', data.get('Columns'))
        self.set_property(v, 'rows', data.get('Rows'))
        self.set_property(v, 'defaultSample', data.get('FieldIndex'))
        self.set_property(v, 'externalIdentifier',
                          data.get('ExternalIdentifier'))
        self.set_property(v, 'status', data.get('Status'))
        v.wellOriginX = self.to_unit(data.get('WellOriginX'))
        v.wellOriginY = self.to_unit(data.get('WellOriginY'))

        for well in data.get('Wells', list()):
            well_decoder = self.ctx.get_decoder(well['@type'])
            v.addWell(well_decoder.decode(well))
        return v


class Plate201606Decoder(Plate201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Plate'


if SCHEMA_VERSION == '2015-01':
    decoder = (Plate201501Decoder.TYPE, Plate201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Plate201606Decoder.TYPE, Plate201606Decoder)
PlateDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import RoiI


class Roi201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#ROI'

    OMERO_CLASS = RoiI

    def decode(self, data):
        v = super(Roi201501Decoder, self).decode(data)
        self.set_property(v, 'name', data.get('Name'))
        self.set_property(v, 'description', data.get('Description'))
        for shape in data.get('shapes', list()):
            shape_decoder = self.ctx.get_decoder(shape['@type'])
            v.addShape(shape_decoder.decode(shape))
        return v


class Roi201606Decoder(Roi201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#ROI'


if SCHEMA_VERSION == '2015-01':
    decoder = (Roi201501Decoder.TYPE, Roi201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Roi201606Decoder.TYPE, Roi201606Decoder)
RoiDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Decoder

try:
    # Import transform classes introduced in OMERO 5.3.0
    from omero.model import AffineTransformI
except ImportError:
    # Use internal AffineTransformI classes for OMERO 5.2.x
    from omero_marshal.legacy.affinetransform import AffineTransformI
from omero.rtypes import rdouble


class AffineTransform201501Decoder(Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#AffineTransform'

    OMERO_CLASS = AffineTransformI

    def decode(self, data):
        t = super(AffineTransform201501Decoder, self).decode(data)
        t.setA00(rdouble(data['A00']))
        t.setA10(rdouble(data['A10']))
        t.setA01(rdouble(data['A01']))
        t.setA11(rdouble(data['A11']))
        t.setA02(rdouble(data['A02']))
        t.setA12(rdouble(data['A12']))
        return t


class AffineTransform201606Decoder(AffineTransform201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#AffineTransform'

    def set_transform(self, data):
        return data


if SCHEMA_VERSION == '2015-01':
    decoder = (AffineTransform201501Decoder.TYPE, AffineTransform201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (AffineTransform201606Decoder.TYPE, AffineTransform201606Decoder)
AffineTransformDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotationDecoder
from omero.model import TermAnnotationI


class TermAnnotation201501Decoder(AnnotationDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01#TermAnnotation'

    OMERO_CLASS = TermAnnotationI

    def decode(self, data):
        v = super(TermAnnotation201501Decoder, self).decode(data)
        self.set_property(v, 'termValue', data.get('Value'))
        return v


class TermAnnotation201606Decoder(TermAnnotation201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#TermAnnotation'


if SCHEMA_VERSION == '2015-01':
    decoder = (TermAnnotation201501Decoder.TYPE,
               TermAnnotation201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (TermAnnotation201606Decoder.TYPE,
               TermAnnotation201606Decoder)
TermAnnotationDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#


# Needed to avoid import errors when this is the first import
import omero.all  # noqa
from omero import RType
from omero_model_UnitBase import UnitBase
from omero.rtypes import unwrap


class Encoder(object):

    TYPE = ''

    def __init__(self, ctx):
        self.ctx = ctx

    def set_if_not_none(self, v, key, value):
        if value is None:
            return
        if isinstance(value, RType):
            v[key] = value.getValue()
        elif isinstance(value, UnitBase):
            self.encode_unit(v, key, value)
        else:
            v[key] = value

    def encode_unit(self, v, key, value):
        v[key] = {
            '@type': 'TBD#%s' % value.__class__.__name__,
            'Unit': value.getUnit().name,
            'Symbol': value.getSymbol(),
            'Value': value.getValue()
        }

    def encode(self, obj):
        v = {'@type': self.TYPE}
        if hasattr(obj, 'id'):
            obj_id = unwrap(obj.id)
            if obj_id is not None:
                v['@id'] = obj_id
        if hasattr(obj, 'details') and obj.details is not None:
            encoder = self.ctx.get_encoder(obj.details.__class__)
            v['omero:details'] = encoder.encode(obj.details)

        return v

    @classmethod
    def rgba_to_int(cls, red, green, blue, alpha):
        """
        Encodes the color as an Integer in RGBA encoding

        Returns None if any of red, green or blue are None.
        If alpha is None we use 255 by default.

        :return:    Integer
        :rtype:     int
        """
        red = unwrap(red)
        green = unwrap(green)
        blue = unwrap(blue)
        alpha = unwrap(alpha)
        if red is None or green is None or blue is None:
            return None
        if alpha is None:
            alpha = 255
        r = red << 24
        g = green << 16
        b = blue << 8
        a = alpha << 0
        rgba_int = r+g+b+a
        if (rgba_int > (2**31-1)):       # convert to signed 32-bit int
            rgba_int = rgba_int - 2**32
        return rgba_int



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import PlateI


class Plate201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#Plate'

    def encode(self, obj):
        v = super(Plate201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'Name', obj.name)
        self.set_if_not_none(v, 'Description', obj.description)
        self.set_if_not_none(v, 'ColumnNamingConvention',
                             obj.columnNamingConvention)
        self.set_if_not_none(v, 'RowNamingConvention',
                             obj.rowNamingConvention)
        self.set_if_not_none(v, 'Columns', obj.columns)
        self.set_if_not_none(v, 'Rows', obj.rows)
        self.set_if_not_none(v, 'FieldIndex', obj.defaultSample)
        self.set_if_not_none(v, 'ExternalIdentifier', obj.externalIdentifier)
        self.set_if_not_none(v, 'Status', obj.status)
        self.set_if_not_none(v, 'WellOriginX', obj.wellOriginX)
        self.set_if_not_none(v, 'WellOriginY', obj.wellOriginY)

        if obj.isWellsLoaded() and obj.sizeOfWells() > 0:
            wells = list()
            for well in obj.copyWells():
                well_encoder = self.ctx.get_encoder(well.__class__)
                wells.append(
                    well_encoder.encode(well)
                )
            v['Wells'] = wells
        return v


class Plate201606Encoder(Plate201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Plate'


if SCHEMA_VERSION == '2015-01':
    encoder = (PlateI, Plate201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (PlateI, Plate201606Encoder)
PlateEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Encoder
from omero.model import PixelsI


class Pixels201501Encoder(Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Pixels'

    def encode(self, obj):
        v = super(Pixels201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'omero:methodology', obj.methodology)
        self.set_if_not_none(v, 'PhysicalSizeX', obj.physicalSizeX)
        self.set_if_not_none(v, 'PhysicalSizeY', obj.physicalSizeY)
        self.set_if_not_none(v, 'PhysicalSizeZ', obj.physicalSizeZ)
        self.set_if_not_none(v, 'omero:sha1', obj.sha1)
        self.set_if_not_none(v, 'SignificantBits', obj.significantBits)
        self.set_if_not_none(v, 'SizeX', obj.sizeX)
        self.set_if_not_none(v, 'SizeY', obj.sizeY)
        self.set_if_not_none(v, 'SizeZ', obj.sizeZ)
        self.set_if_not_none(v, 'SizeC', obj.sizeC)
        self.set_if_not_none(v, 'SizeT', obj.sizeT)
        self.set_if_not_none(v, 'TimeIncrement', obj.timeIncrement)
        self.set_if_not_none(v, 'omero:waveIncrement', obj.waveIncrement)
        self.set_if_not_none(v, 'omero:waveStart', obj.waveStart)
        dimension_order = obj.dimensionOrder
        if dimension_order is not None and dimension_order.isLoaded():
            dimension_order_encoder = \
                self.ctx.get_encoder(dimension_order.__class__)
            v['DimensionOrder'] = \
                dimension_order_encoder.encode(dimension_order)
        pixels_type = obj.pixelsType
        if pixels_type is not None and pixels_type.isLoaded():
            pixels_type_encoder = \
                self.ctx.get_encoder(pixels_type.__class__)
            v['Type'] = pixels_type_encoder.encode(pixels_type)
        if obj.isChannelsLoaded() and obj.sizeOfChannels() > 0:
            channels = list()
            for channel in obj.copyChannels():
                channel_encoder = self.ctx.get_encoder(channel.__class__)
                channels.append(channel_encoder.encode(channel))
            v['Channels'] = channels
        return v


class Pixels201606Encoder(Pixels201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Pixels'


if SCHEMA_VERSION == '2015-01':
    encoder = (PixelsI, Pixels201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (PixelsI, Pixels201606Encoder)
PixelsEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from .. import Encoder


class EnumEncoder(Encoder):

    def __init__(self, ctx):
        super(EnumEncoder, self).__init__(ctx)

    def encode(self, obj):
        v = super(EnumEncoder, self).encode(obj)
        self.set_if_not_none(v, 'value', obj.value)
        return v



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Decoder
from omero.model import PixelsI


class Pixels201501Decoder(Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Pixels'

    OMERO_CLASS = PixelsI

    def decode(self, data):
        v = super(Pixels201501Decoder, self).decode(data)
        self.set_property(v, 'methodology', data.get('omero:methodology'))
        v.physicalSizeX = self.to_unit(data.get('PhysicalSizeX'))
        v.physicalSizeY = self.to_unit(data.get('PhysicalSizeY'))
        v.physicalSizeZ = self.to_unit(data.get('PhysicalSizeZ'))
        self.set_property(v, 'sha1', data.get('omero:sha1'))
        self.set_property(v, 'significantBits', data.get('SignificantBits'))
        self.set_property(v, 'sizeX', data.get('SizeX'))
        self.set_property(v, 'sizeY', data.get('SizeY'))
        self.set_property(v, 'sizeZ', data.get('SizeZ'))
        self.set_property(v, 'sizeC', data.get('SizeC'))
        self.set_property(v, 'sizeT', data.get('SizeT'))
        v.timeIncrement = self.to_unit(data.get('TimeIncrement'))
        self.set_property(v, 'waveIncrement', data.get('omero:waveIncrement'))
        self.set_property(v, 'waveStart', data.get('omero:waveStart'))
        dimension_order = data.get('DimensionOrder')
        if dimension_order is not None:
            dimension_order_decoder = \
                self.ctx.get_decoder(dimension_order['@type'])
            v.dimensionOrder = dimension_order_decoder.decode(dimension_order)
        pixels_type = data.get('Type')
        if pixels_type is not None:
            pixels_type_decoder = \
                self.ctx.get_decoder(pixels_type['@type'])
            v.pixelsType = pixels_type_decoder.decode(pixels_type)
        channels = data.get('Channels', list())
        for channel in channels:
            channel_decoder = self.ctx.get_decoder(channel['@type'])
            v.addChannel(channel_decoder.decode(channel))
        return v


class Pixels201606Decoder(Pixels201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Pixels'


if SCHEMA_VERSION == '2015-01':
    decoder = (Pixels201501Decoder.TYPE, Pixels201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Pixels201606Decoder.TYPE, Pixels201606Decoder)
PixelsDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from .. import Decoder


class EnumDecoder(Decoder):

    def decode(self, data):
        v = super(EnumDecoder, self).decode(data)
        self.set_property(v, 'value', data.get('value'))
        return v



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .shape import ShapeEncoder
from omero.model import LineI


class Line201501Encoder(ShapeEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#Line'

    def encode(self, obj):
        v = super(Line201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'X1', obj.x1)
        self.set_if_not_none(v, 'Y1', obj.y1)
        self.set_if_not_none(v, 'X2', obj.x2)
        self.set_if_not_none(v, 'Y2', obj.y2)
        return v


class Line201606Encoder(Line201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Line'

    def encode(self, obj):
        v = super(Line201606Encoder, self).encode(obj)
        self.set_if_not_none(v, 'MarkerStart', obj.markerStart)
        self.set_if_not_none(v, 'MarkerEnd', obj.markerEnd)
        return v


if SCHEMA_VERSION == '2015-01':
    encoder = (LineI, Line201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (LineI, Line201606Encoder)
LineEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import ImageI


class Image201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Image'

    OMERO_CLASS = ImageI

    def decode(self, data):
        v = super(Image201501Decoder, self).decode(data)
        self.set_property(v, 'acquisitionDate', data.get('AcquisitionDate'))
        self.set_property(v, 'archived', data.get('omero:archived'))
        self.set_property(v, 'description', data.get('Description'))
        self.set_property(v, 'name', data.get('Name'))
        self.set_property(v, 'partial', data.get('omero:partial'))
        self.set_property(v, 'series', data.get('omero:series'))
        _format = data.get('omero:format')
        if _format is not None:
            format_decoder = self.ctx.get_decoder(_format['@type'])
            v.format = format_decoder.decode(_format)
        pixels = data.get('Pixels')
        if pixels is not None:
            pixels_decoder = self.ctx.get_decoder(pixels['@type'])
            v.addPixels(pixels_decoder.decode(pixels))
        return v


class Image201606Decoder(Image201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Image'


if SCHEMA_VERSION == '2015-01':
    decoder = (Image201501Decoder.TYPE, Image201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Image201606Decoder.TYPE, Image201606Decoder)
ImageDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import PlateAcquisitionI


class PlateAcquisition201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#PlateAcquisition'

    OMERO_CLASS = PlateAcquisitionI

    def decode(self, data):
        v = super(PlateAcquisition201501Decoder, self).decode(data)
        self.set_property(v, 'name', data.get('Name'))
        self.set_property(v, 'description', data.get('Description'))
        self.set_property(
            v, 'maximumFieldCount', data.get('MaximumFieldCount')
        )
        self.set_property(v, 'startTime', data.get('StartTime'))
        self.set_property(v, 'endTime', data.get('EndTime'))
        return v


class PlateAcquisition201606Decoder(PlateAcquisition201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#PlateAcquisition'


if SCHEMA_VERSION == '2015-01':
    decoder = (PlateAcquisition201501Decoder.TYPE,
               PlateAcquisition201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (PlateAcquisition201606Decoder.TYPE,
               PlateAcquisition201606Decoder)
PlateAcquisitionDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import DatasetI


class Dataset201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Dataset'

    def encode(self, obj):
        v = super(Dataset201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'Name', obj.name)
        self.set_if_not_none(v, 'Description', obj.description)
        if obj.isImageLinksLoaded() and obj.sizeOfImageLinks() > 0:
            images = list()
            for image_link in obj.copyImageLinks():
                image = image_link.child
                image_encoder = self.ctx.get_encoder(image.__class__)
                images.append(
                    image_encoder.encode(image)
                )
            v['Images'] = images
        return v


class Dataset201606Encoder(Dataset201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Dataset'


if SCHEMA_VERSION == '2015-01':
    encoder = (DatasetI, Dataset201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (DatasetI, Dataset201606Encoder)
DatasetEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotationDecoder
from omero.model import DoubleAnnotationI


class DoubleAnnotation201501Decoder(AnnotationDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01#DoubleAnnotation'

    OMERO_CLASS = DoubleAnnotationI

    def decode(self, data):
        v = super(DoubleAnnotation201501Decoder, self).decode(data)
        self.set_property(v, 'doubleValue', data.get('Value'))
        return v


class DoubleAnnotation201606Decoder(DoubleAnnotation201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#' \
        'DoubleAnnotation'


if SCHEMA_VERSION == '2015-01':
    decoder = (DoubleAnnotation201501Decoder.TYPE,
               DoubleAnnotation201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (DoubleAnnotation201606Decoder.TYPE,
               DoubleAnnotation201606Decoder)
DoubleAnnotationDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import ImageI


class Image201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Image'

    def encode(self, obj):
        v = super(Image201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'AcquisitionDate', obj.acquisitionDate)
        self.set_if_not_none(v, 'omero:archived', obj.archived)
        self.set_if_not_none(v, 'Description', obj.description)
        self.set_if_not_none(v, 'Name', obj.name)
        self.set_if_not_none(v, 'omero:partial', obj.partial)
        self.set_if_not_none(v, 'omero:series', obj.series)
        _format = obj.format
        if _format is not None and _format.isLoaded():
            format_encoder = self.ctx.get_encoder(obj.format.__class__)
            self.set_if_not_none(
                v, 'omero:format', format_encoder.encode(obj.format)
            )
        if obj.isPixelsLoaded() and obj.sizeOfPixels() > 0:
            pixels = obj.getPrimaryPixels()
            pixels_encoder = self.ctx.get_encoder(pixels.__class__)
            v['Pixels'] = pixels_encoder.encode(pixels)
        return v


class Image201606Encoder(Image201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Image'


if SCHEMA_VERSION == '2015-01':
    encoder = (ImageI, Image201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (ImageI, Image201606Encoder)
ImageEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .shape import ShapeEncoder
from omero.model import PointI


class Point201501Encoder(ShapeEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#Point'
    X_PROPERTY_NAME = 'cx'
    Y_PROPERTY_NAME = 'cy'

    def encode(self, obj):
        v = super(Point201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'X', getattr(obj, self.X_PROPERTY_NAME))
        self.set_if_not_none(v, 'Y', getattr(obj, self.Y_PROPERTY_NAME))
        return v


class Point201606Encoder(Point201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Point'
    X_PROPERTY_NAME = 'x'
    Y_PROPERTY_NAME = 'y'


if SCHEMA_VERSION == '2015-01':
    encoder = (PointI, Point201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (PointI, Point201606Encoder)
PointEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from .. import Decoder
from omero.model import PermissionsI


class PermissionsDecoder(Decoder):

    TYPE = 'TBD#Permissions'

    OMERO_CLASS = PermissionsI

    def decode(self, data):
        o = PermissionsI()
        o.from_string(data['perm'])
        o._restrictions = [
            not data['canLink'], not data['canEdit'],
            not data['canDelete'], not data['canAnnotate']
        ]
        return o


decoder = (PermissionsDecoder.TYPE, PermissionsDecoder)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Encoder
from omero.model import WellSampleI


class WellSample201501Encoder(Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#WellSample'

    def encode(self, obj):
        v = super(WellSample201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'PositionX', obj.posX)
        self.set_if_not_none(v, 'PositionY', obj.posY)
        self.set_if_not_none(v, 'Timepoint', obj.timepoint)
        if obj.image is not None and obj.image.isLoaded():
            image_encoder = self.ctx.get_encoder(obj.image.__class__)
            v['Image'] = image_encoder.encode(obj.image)

        if obj.plateAcquisition is not None \
                and obj.plateAcquisition.isLoaded():
            encoder = self.ctx.get_encoder(obj.plateAcquisition.__class__)
            v['PlateAcquisition'] = encoder.encode(obj.plateAcquisition)
        return v


class WellSample201606Encoder(WellSample201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#WellSample'


if SCHEMA_VERSION == '2015-01':
    encoder = (WellSampleI, WellSample201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (WellSampleI, WellSample201606Encoder)
WellSampleEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import RoiI


class Roi201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#ROI'

    def encode(self, obj):
        v = super(Roi201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'Name', obj.name)
        self.set_if_not_none(v, 'Description', obj.description)
        if obj.isShapesLoaded() and obj.sizeOfShapes() > 0:
            shapes = list()
            for shape in obj.copyShapes():
                shape_encoder = self.ctx.get_encoder(shape.__class__)
                shapes.append(shape_encoder.encode(shape))
            v['shapes'] = shapes
        return v


class Roi201606Encoder(Roi201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#ROI'


if SCHEMA_VERSION == '2015-01':
    encoder = (RoiI, Roi201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (RoiI, Roi201606Encoder)
RoiEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Decoder
from omero.model import ExperimenterI


class Experimenter201501Decoder(Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Experimenter'

    OMERO_CLASS = ExperimenterI

    def decode(self, data):
        v = super(Experimenter201501Decoder, self).decode(data)
        self.set_property(v, 'firstName', data.get('FirstName'))
        self.set_property(v, 'middleName', data.get('MiddleName'))
        self.set_property(v, 'lastName', data.get('LastName'))
        self.set_property(v, 'email', data.get('Email'))
        self.set_property(v, 'institution', data.get('Institution'))
        self.set_property(v, 'omeName', data.get('UserName'))
        return v


class Experimenter201606Decoder(Experimenter201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Experimenter'


if SCHEMA_VERSION == '2015-01':
    decoder = (Experimenter201501Decoder.TYPE, Experimenter201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Experimenter201606Decoder.TYPE, Experimenter201606Decoder)
ExperimenterDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Encoder
from omero.model import ExperimenterI


class Experimenter201501Encoder(Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2015-01#Experimenter'

    def encode(self, obj):
        v = super(Experimenter201501Encoder, self).encode(obj)
        if not obj.isLoaded():
            return v
        self.set_if_not_none(v, 'FirstName', obj.firstName)
        self.set_if_not_none(v, 'MiddleName', obj.middleName)
        self.set_if_not_none(v, 'LastName', obj.lastName)
        self.set_if_not_none(v, 'Email', obj.email)
        self.set_if_not_none(v, 'Institution', obj.institution)
        self.set_if_not_none(v, 'UserName', obj.omeName)
        return v


class Experimenter201606Encoder(Experimenter201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Experimenter'


if SCHEMA_VERSION == '2015-01':
    encoder = (ExperimenterI, Experimenter201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (ExperimenterI, Experimenter201606Encoder)
ExperimenterEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .text_annotation import TextAnnotationDecoder
from omero.model import XmlAnnotationI


class XmlAnnotation201501Decoder(TextAnnotationDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01#XmlAnnotation'

    OMERO_CLASS = XmlAnnotationI

    def decode(self, data):
        v = super(XmlAnnotation201501Decoder, self).decode(data)
        return v


class XmlAnnotation201606Decoder(XmlAnnotation201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#' \
        'XmlAnnotation'


if SCHEMA_VERSION == '2015-01':
    decoder = (XmlAnnotation201501Decoder.TYPE,
               XmlAnnotation201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (XmlAnnotation201606Decoder.TYPE,
               XmlAnnotation201606Decoder)
XmlAnnotationDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .shape import ShapeDecoder


# Handle differences in class naming between OMERO 5.1.x and 5.2.x
try:
    # OMERO 5.1.x
    from omero.model import RectI as RectangleI
except ImportError:
    # OMERO 5.2.x
    from omero.model import RectangleI


class Rectangle201501Decoder(ShapeDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#Rectangle'

    OMERO_CLASS = RectangleI

    def decode(self, data):
        v = super(Rectangle201501Decoder, self).decode(data)
        self.set_property(v, 'x', data.get('X'))
        self.set_property(v, 'y', data.get('Y'))
        self.set_property(v, 'width', data.get('Width'))
        self.set_property(v, 'height', data.get('Height'))
        return v


class Rectangle201606Decoder(Rectangle201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Rectangle'


if SCHEMA_VERSION == '2015-01':
    decoder = (Rectangle201501Decoder.TYPE, Rectangle201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Rectangle201606Decoder.TYPE, Rectangle201606Decoder)
RectangleDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from math import sin, cos, radians
from omero.rtypes import unwrap


class AffineTransformI(object):
    """
    Class handling OMERO 5.1 and OMERO 5.2 transform string representations.
    Its fields and methods are directly mirroring the
    omero.model.AffineTransform class introduced in OMERO 5.3 to reduce the
    complexity of the transform encoders/decoders.
    """

    def __init__(self, *args):
        # Set the id to -1 to differentiate the marshalled transform from
        # objects
        # Setting the id also allows the transform decoder to inherit the
        # superclass decode() method
        self.id = -1
        self._svg_transform = None
        self._a00 = None
        self._a10 = None
        self._a01 = None
        self._a11 = None
        self._a02 = None
        self._a12 = None

    def convert_svg_transform(self, transform):
        """
        Converts a string representing a SVG transform into
        AffineTransform fields.
        See https://www.w3.org/TR/SVG/coords.html#TransformAttribute for the
        specification of the transform strings. skewX and skewY are not
        supported.
        Raises:
            ValueError: If transform is not a valid and supported SVG
            transform.
        """

        tr, args = transform[:-1].split('(')
        a = [float(arg) for arg in args.split(' ')]

        # Handle various string tranformations
        if tr == 'matrix':
            pass
        elif tr == 'translate':
            a = [1.0, 0.0, 0.0, 1.0, a[0], a[1] if len(a) > 1 else 0.0]
        elif tr == 'scale':
            a = [a[0], 0.0, 0.0, a[-1], 0.0, 0.0]
        elif tr == 'rotate':
            x = a[1] if len(a) > 1 else 0.0
            y = a[2] if len(a) > 1 else 0.0
            rad = radians(a[0])
            s = sin(rad)
            c = cos(rad)
            a = [
                c,
                s,
                -s,
                c,
                x * (1 - c) + y * s,
                -x * s + y * (1 - c),
            ]
        else:
            raise ValueError('Unknown transformation "%s"' % transform)

        self._svg_transform = transform
        self._a00 = a[0]
        self._a10 = a[1]
        self._a01 = a[2]
        self._a11 = a[3]
        self._a02 = a[4]
        self._a12 = a[5]

    def __str__(self):
        """Returns a string matrix representation of the transform"""
        return 'matrix(%s)' % ' '.join(map(str, [
            unwrap(self._a00),
            unwrap(self._a10),
            unwrap(self._a01),
            unwrap(self._a11),
            unwrap(self._a02),
            unwrap(self._a12),
            ]))

    @property
    def svg_transform(self):
        """Retrieves the string containing the SVG transform"""
        return self._svg_transform

    def getA00(self):
        return self._a00

    def getA10(self):
        return self._a10

    def getA01(self):
        return self._a01

    def getA11(self):
        return self._a11

    def getA02(self):
        return self._a02

    def getA12(self):
        return self._a12

    def setA00(self, value):
        self._a00 = value

    def setA10(self, value):
        self._a10 = value

    def setA01(self, value):
        self._a01 = value

    def setA11(self, value):
        self._a11 = value

    def setA02(self, value):
        self._a02 = value

    def setA12(self, value):
        self._a12 = value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .shape import ShapeDecoder
from omero.model import MaskI


class Mask201501Decoder(ShapeDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#Mask'

    OMERO_CLASS = MaskI

    def decode(self, data):
        v = super(Mask201501Decoder, self).decode(data)
        self.set_property(v, 'x', data.get('X'))
        self.set_property(v, 'y', data.get('Y'))
        self.set_property(v, 'width', data.get('Width'))
        self.set_property(v, 'height', data.get('Height'))
        return v


class Mask201606Decoder(Mask201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Mask'


if SCHEMA_VERSION == '2015-01':
    decoder = (Mask201501Decoder.TYPE, Mask201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Mask201606Decoder.TYPE, Mask201606Decoder)
MaskDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

import importlib
import pkgutil
import logging
import re
from .encode import encoders
from .decode import decoders
from distutils.version import StrictVersion
from omero_version import omero_version

logger = logging.getLogger('omero-marshal')


ENCODERS = dict()
DECODERS = dict()


def dumps(obj):
    pass


def get_encoder(t):
    try:
        return ENCODERS[t]
    except KeyError:
        logger.warn('Requested unknown encoder %s' % t, exc_info=True)
        return None


def get_decoder(t):
    try:
        return DECODERS[t]
    except KeyError:
        logger.warn('Requested unknown decoder %s' % t, exc_info=True)
        return None


VERSION_REGEXP = re.compile(r'^(\d+\.\d+)\.(a|b|dev|rc)?\d+')


def get_schema_version(version):
    m = VERSION_REGEXP.search(version)
    if m is None:
        raise Exception("Invalid OMERO version number: " + version)
    v = StrictVersion(m.group(1))
    if v >= StrictVersion('5.1.0') and v < StrictVersion('5.3.0'):
        return '2015-01'
    elif v >= StrictVersion('5.3.0') and v < StrictVersion('6.0.0'):
        return '2016-06'
    else:
        raise Exception("Unsupported OMERO version: " + version)


SCHEMA_VERSION = get_schema_version(omero_version)
BASE_URL = 'http://www.openmicroscopy.org/Schemas'
if SCHEMA_VERSION == "2015-01":
    ROI_NS = 'ROI'
    SA_NS = 'SA'
    SPW_NS = 'SPW'
else:
    ROI_NS = 'OME'
    SA_NS = 'OME'
    SPW_NS = 'OME'
OME_SCHEMA_URL = '%s/OME/%s' % (BASE_URL, SCHEMA_VERSION)
ROI_SCHEMA_URL = '%s/%s/%s' % (BASE_URL, ROI_NS, SCHEMA_VERSION)
SA_SCHEMA_URL = '%s/%s/%s' % (BASE_URL, SA_NS, SCHEMA_VERSION)
SPW_SCHEMA_URL = '%s/%s/%s' % (BASE_URL, SPW_NS, SCHEMA_VERSION)


class MarshallingCtx(object):

    def __init__(self, encoders, decoders):
        self.encoders = encoders
        self.decoders = decoders

    def get_encoder(self, t):
        try:
            return self.encoders[t]
        except KeyError:
            logger.warn('Requested unknown encoder %s' % t, exc_info=True)
            return None

    def get_decoder(self, t):
        try:
            return self.decoders[t]
        except KeyError:
            logger.warn('Requested unknown decoder %s' % t, exc_info=True)
            return None


_ctx = MarshallingCtx(ENCODERS, DECODERS)

packages = pkgutil.walk_packages(encoders.__path__, encoders.__name__ + '.')
for module_loader, name, ispkg in packages:
    m = importlib.import_module(name)
    try:
        t, encoder = m.encoder
        ENCODERS[t] = encoder(_ctx)
    except AttributeError:
        pass

packages = pkgutil.walk_packages(decoders.__path__, decoders.__name__ + '.')
for module_loader, name, ispkg in packages:
    m = importlib.import_module(name)
    try:
        t, decoder = m.decoder
        DECODERS[t] = decoder(_ctx)
    except AttributeError:
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import Shape
from omero.rtypes import unwrap


class Shape201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#Shape'

    def encode(self, obj):
        v = super(Shape201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'FillColor', obj.fillColor)
        self.set_if_not_none(v, 'FillRule', obj.fillRule)
        self.set_if_not_none(v, 'FontFamily', obj.fontFamily)
        self.set_if_not_none(v, 'FontSize', obj.fontSize)
        self.set_if_not_none(v, 'FontStyle', obj.fontStyle)
        self.set_if_not_none(v, 'Locked', obj.locked)
        self.set_if_not_none(v, 'StrokeColor', obj.strokeColor)
        self.set_if_not_none(v, 'StrokeDashArray', obj.strokeDashArray)
        self.set_if_not_none(v, 'StrokeWidth', obj.strokeWidth)
        self.set_if_not_none(v, 'Text', obj.textValue)
        self.set_if_not_none(v, 'TheC', obj.theC)
        self.set_if_not_none(v, 'TheT', obj.theT)
        self.set_if_not_none(v, 'TheZ', obj.theZ)
        transform = self.get_transform(obj.transform)
        if transform:
            transform_encoder = self.ctx.get_encoder(transform.__class__)
            self.set_if_not_none(
                v, 'Transform', transform_encoder.encode(transform))
        self.set_linecap(v, obj)
        self.set_visible(v, obj)
        return v

    def set_linecap(self, v, obj):
        self.set_if_not_none(v, 'LineCap', obj.strokeLineCap)

    def set_visible(self, v, obj):
        self.set_if_not_none(v, 'Visible', obj.visibility)

    def get_transform(self, transform):
        transform = unwrap(transform)
        if not transform:
            return

        # For OMERO 5.1.x and OMERO 5.2.x the unwrapped transform is a string.
        # To facilitate the encoding we construct an internal AffineTransform
        # object and use convert_svg_transform() to map the SVG string
        # representation into the fields defined by the schema.
        from omero_marshal.legacy.affinetransform import AffineTransformI
        t = AffineTransformI()
        try:
            t.convert_svg_transform(transform)
        except ValueError:
            # Means the string is an invalid or unsupported SVG transform
            return
        return t


class Shape201606Encoder(Shape201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Shape'

    def set_linecap(self, v, obj):
        pass

    def set_visible(self, v, obj):
        pass

    def get_transform(self, transform):
        if not transform:
            return None
        if (transform.getA00() is None and transform.getA10() is None and
                transform.getA01() is None and transform.getA11() is None and
                transform.getA02() is None and transform.getA12() is None):
            return None
        return transform


if SCHEMA_VERSION == '2015-01':
    encoder = (Shape, Shape201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (Shape, Shape201606Encoder)
ShapeEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .shape import ShapeDecoder
from omero.model import EllipseI


class Ellipse201501Decoder(ShapeDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#Ellipse'

    OMERO_CLASS = EllipseI
    X_PROPERTY_NAME = 'cx'
    Y_PROPERTY_NAME = 'cy'
    RADIUSX_PROPERTY_NAME = 'rx'
    RADIUSY_PROPERTY_NAME = 'ry'

    def decode(self, data):
        v = super(Ellipse201501Decoder, self).decode(data)
        self.set_property(v, self.X_PROPERTY_NAME, data.get('X'))
        self.set_property(v, self.Y_PROPERTY_NAME, data.get('Y'))
        self.set_property(v, self.RADIUSX_PROPERTY_NAME, data.get('RadiusX'))
        self.set_property(v, self.RADIUSY_PROPERTY_NAME, data.get('RadiusY'))
        return v


class Ellipse201606Decoder(Ellipse201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Ellipse'
    X_PROPERTY_NAME = 'x'
    Y_PROPERTY_NAME = 'y'
    RADIUSX_PROPERTY_NAME = 'radiusX'
    RADIUSY_PROPERTY_NAME = 'radiusY'


if SCHEMA_VERSION == '2015-01':
    decoder = (Ellipse201501Decoder.TYPE, Ellipse201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Ellipse201606Decoder.TYPE, Ellipse201606Decoder)
EllipseDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .shape import ShapeDecoder
from omero.model import PointI


class Point201501Decoder(ShapeDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/ROI/2015-01#Point'

    OMERO_CLASS = PointI
    X_PROPERTY_NAME = 'cx'
    Y_PROPERTY_NAME = 'cy'

    def decode(self, data):
        v = super(Point201501Decoder, self).decode(data)
        self.set_property(v, self.X_PROPERTY_NAME, data.get('X'))
        self.set_property(v, self.Y_PROPERTY_NAME, data.get('Y'))
        return v


class Point201606Decoder(Point201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Point'
    X_PROPERTY_NAME = 'x'
    Y_PROPERTY_NAME = 'y'


if SCHEMA_VERSION == '2015-01':
    decoder = (Point201501Decoder.TYPE, Point201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Point201606Decoder.TYPE, Point201606Decoder)
PointDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotationEncoder
from omero.model import TimestampAnnotationI


class TimestampAnnotation201501Encoder(AnnotationEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01' \
        '#TimestampAnnotation'

    def encode(self, obj):
        v = super(TimestampAnnotation201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'Value', obj.timeValue)
        return v


class TimestampAnnotation201606Encoder(TimestampAnnotation201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06' \
        '#TimestampAnnotation'


if SCHEMA_VERSION == '2015-01':
    encoder = (TimestampAnnotationI, TimestampAnnotation201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (TimestampAnnotationI, TimestampAnnotation201606Encoder)
TimestampAnnotationEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableEncoder
from omero.model import WellI


class Well201501Encoder(AnnotatableEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#Well'

    def encode(self, obj):
        v = super(Well201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'Column', obj.column)
        self.set_if_not_none(v, 'Row', obj.row)
        self.set_if_not_none(v, 'ExternalDescription', obj.externalDescription)
        self.set_if_not_none(v, 'ExternalIdentifier', obj.externalIdentifier)
        self.set_if_not_none(v, 'Type', obj.type)
        color = self.rgba_to_int(obj.red, obj.green, obj.blue, obj.alpha)
        self.set_if_not_none(v, 'Color', color)
        self.set_if_not_none(v, 'omero:status', obj.status)

        if obj.isWellSamplesLoaded() and obj.sizeOfWellSamples() > 0:
            wellsamples = list()
            for wellsample in obj.copyWellSamples():
                if wellsample is None:
                    continue
                wellsample_encoder = self.ctx.get_encoder(wellsample.__class__)
                wellsamples.append(
                    wellsample_encoder.encode(wellsample)
                )
            v['WellSamples'] = wellsamples
        return v


class Well201606Encoder(Well201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Well'


if SCHEMA_VERSION == '2015-01':
    encoder = (WellI, Well201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (WellI, Well201606Encoder)
WellEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import WellI


class Well201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#Well'

    OMERO_CLASS = WellI

    def decode(self, data):
        v = super(Well201501Decoder, self).decode(data)
        self.set_property(v, 'column', data.get('Column'))
        self.set_property(v, 'row', data.get('Row'))
        self.set_property(
            v, 'externalDescription', data.get('ExternalDescription')
        )
        self.set_property(
            v, 'externalIdentifier', data.get('ExternalIdentifier')
        )
        self.set_property(v, 'type', data.get('Type'))
        color = data.get('Color')
        red, green, blue, alpha = self.int_to_rgba(color)
        self.set_property(v, 'red', red)
        self.set_property(v, 'green', green)
        self.set_property(v, 'blue', blue)
        self.set_property(v, 'alpha', alpha)
        self.set_property(v, 'status', data.get('omero:status'))

        for wellsample in data.get('WellSamples', list()):
            wellsample_decoder = self.ctx.get_decoder(wellsample['@type'])
            v.addWellSample(wellsample_decoder.decode(wellsample))
        return v


class Well201606Decoder(Well201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Well'


if SCHEMA_VERSION == '2015-01':
    decoder = (Well201501Decoder.TYPE, Well201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Well201606Decoder.TYPE, Well201606Decoder)
WellDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .. import Encoder
from omero.model import Annotation


class Annotation201501Encoder(Encoder):
    '''
    Annotation
        BasicAnnotation
            BooleanAnnotation
                BooleanAnnotationI
            NumericAnnotation
                DoubleAnnotation
                    DoubleAnnotationI
                LongAnnotation
                    LongAnnotationI
            TermAnnotation
                TermAnnotationI
            TimestampAnnotation
                TimestampAnnotationI
        ListAnnotation
            ListAnnotationI
        MapAnnotation
            MapAnnotationI
        TextAnnotation
            CommentAnnotation
                CommentAnnotationI
            TagAnnotation
                TagAnnotationI
            XmlAnnotation
                XmlAnnotationI
        TypeAnnotation
            FileAnnotation
                FileAnnotationI
    '''

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01#Annotation'

    def encode(self, obj):
        v = super(Annotation201501Encoder, self).encode(obj)
        self.set_if_not_none(v, 'Description', obj.description)
        self.set_if_not_none(v, 'Namespace', obj.ns)
        return v


class Annotation201606Encoder(Annotation201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Annotation'


class Annotatable201501Encoder(Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01#AnnotationRef'

    def encode(self, obj):
        v = super(Annotatable201501Encoder, self).encode(obj)
        if obj.isAnnotationLinksLoaded() and obj.sizeOfAnnotationLinks() > 0:
            annotations = list()
            for annotation_link in obj.copyAnnotationLinks():
                annotation = annotation_link.child
                annotation_encoder = self.ctx.get_encoder(annotation.__class__)
                annotations.append(
                    annotation_encoder.encode(annotation)
                )
            v['Annotations'] = annotations
        return v


class Annotatable201606Encoder(Annotatable201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#AnnotationRef'


if SCHEMA_VERSION == '2015-01':
    encoder = (Annotation, Annotation201501Encoder)
    AnnotatableEncoder = Annotatable201501Encoder
elif SCHEMA_VERSION == '2016-06':
    encoder = (Annotation, Annotation201606Encoder)
    AnnotatableEncoder = Annotatable201606Encoder
AnnotationEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotationEncoder
from omero.model import MapAnnotationI


class MapAnnotation201501Encoder(AnnotationEncoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SA/2015-01' \
        '#MapAnnotation'

    def encode(self, obj):
        v = super(MapAnnotation201501Encoder, self).encode(obj)
        if obj.mapValue is None:
            return None
        self.set_if_not_none(
            v, 'Value', [
                [nv.name, nv.value] for nv in obj.getMapValue()
            ]
        )
        return v


class MapAnnotation201606Encoder(MapAnnotation201501Encoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06' \
        '#MapAnnotation'


if SCHEMA_VERSION == '2015-01':
    encoder = (MapAnnotationI, MapAnnotation201501Encoder)
elif SCHEMA_VERSION == '2016-06':
    encoder = (MapAnnotationI, MapAnnotation201606Encoder)
MapAnnotationEncoder = encoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from .. import Encoder
from omero.model import PermissionsI


class PermissionsEncoder(Encoder):

    TYPE = 'TBD#Permissions'

    def encode(self, obj):
        v = super(PermissionsEncoder, self).encode(obj)
        v['perm'] = str(obj)
        v['canAnnotate'] = obj.canAnnotate()
        v['canDelete'] = obj.canDelete()
        v['canEdit'] = obj.canEdit()
        v['canLink'] = obj.canLink()
        v['isWorldWrite'] = obj.isWorldWrite()
        v['isWorldRead'] = obj.isWorldRead()
        v['isGroupWrite'] = obj.isGroupWrite()
        v['isGroupRead'] = obj.isGroupRead()
        v['isGroupAnnotate'] = obj.isGroupAnnotate()
        v['isUserWrite'] = obj.isUserWrite()
        v['isUserRead'] = obj.isUserRead()
        return v


encoder = (PermissionsI, PermissionsEncoder)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from ... import SCHEMA_VERSION
from .annotation import AnnotatableDecoder
from omero.model import ScreenI


class Screen201501Decoder(AnnotatableDecoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/SPW/2015-01#Screen'

    OMERO_CLASS = ScreenI

    def decode(self, data):
        v = super(Screen201501Decoder, self).decode(data)
        self.set_property(v, 'name', data.get('Name'))
        self.set_property(v, 'description', data.get('Description'))
        self.set_property(v, 'protocolDescription',
                          data.get('ProtocolDescription'))
        self.set_property(v, 'protocolIdentifier',
                          data.get('ProtocolIdentifier'))
        self.set_property(v, 'reagentSetDescription',
                          data.get('ReagentSetDescription'))
        self.set_property(v, 'reagentSetIdentifier',
                          data.get('ReagentSetIdentifier'))
        self.set_property(v, 'type', data.get('Type'))
        for plate in data.get('Plates', list()):
            plate_decoder = self.ctx.get_decoder(plate['@type'])
            v.linkPlate(plate_decoder.decode(plate))
        return v


class Screen201606Decoder(Screen201501Decoder):

    TYPE = 'http://www.openmicroscopy.org/Schemas/OME/2016-06#Screen'


if SCHEMA_VERSION == '2015-01':
    decoder = (Screen201501Decoder.TYPE, Screen201501Decoder)
elif SCHEMA_VERSION == '2016-06':
    decoder = (Screen201606Decoder.TYPE, Screen201606Decoder)
ScreenDecoder = decoder[1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from .. import Decoder
from omero.model import DetailsI


class DetailsDecoder(Decoder):

    TYPE = 'TBD#Details'

    OMERO_CLASS = DetailsI

    def decode(self, data):
        v = DetailsI()
        owner = data.get('owner')
        if owner is not None:
            decoder = self.ctx.get_decoder(owner['@type'])
            v.owner = decoder.decode(owner)
        group = data.get('group')
        if group is not None:
            decoder = self.ctx.get_decoder(group['@type'])
            v.group = decoder.decode(group)
        permissions = data.get('permissions')
        if permissions is not None:
            decoder = self.ctx.get_decoder(permissions['@type'])
            v.permissions = decoder.decode(permissions)
        externalInfo = data.get('externalInfo')
        if externalInfo is not None:
            decoder = self.ctx.get_decoder(externalInfo['@type'])
            v.externalInfo = decoder.decode(externalInfo)
        return v


decoder = (DetailsDecoder.TYPE, DetailsDecoder)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from .enum import EnumEncoder
from omero.model import ContrastMethodI


class ContrastMethodEncoder(EnumEncoder):

    TYPE = 'TBD#ContrastMethod'


encoder = (ContrastMethodI, ContrastMethodEncoder)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# jason@glencoesoftware.com.
#

from .. import Encoder
from omero.model import DetailsI


class DetailsEncoder(Encoder):

    TYPE = 'TBD#Details'

    def encode(self, obj):
        v = super(DetailsEncoder, self).encode(obj)
        if obj.owner is not None:
            encoder = self.ctx.get_encoder(obj.owner.__class__)
            v['owner'] = encoder.encode(obj.owner)
        if obj.group is not None:
            encoder = self.ctx.get_encoder(obj.group.__class__)
            v['group'] = encoder.encode(obj.group)
        if obj.permissions is not None:
            encoder = self.ctx.get_encoder(obj.permissions.__class__)
            v['permissions'] = encoder.encode(obj.permissions)
        if obj.externalInfo is not None:
            encoder = self.ctx.get_encoder(obj.externalInfo.__class__)
            v['externalInfo'] = encoder.encode(obj.externalInfo)
        return v


encoder = (DetailsI, DetailsEncoder)
